#ifndef MATRIX_H
#define MATRIX_H

#define MX_OK		0
#define MX_UNSET	1
#define MX_NOVAL	2
#define MX_NOMEM	3
#define MX_USERR	4

struct _matrix {
	double *val;
	unsigned short rows;
	unsigned short cols;
};

typedef struct _matrix matrix;

/* function constructss a new matrix */
matrix *matrix_new();

/* funtion destructs an existing matrix */
int mx_free(matrix **mx);

/* function returns information about matrix */
int mx_state(const matrix *mx);

/* function sets val as matrix value, if possible */
int mx_set(const double *src, matrix *mx, unsigned short rows, unsigned short cols);

/* function sets row r of matrix mx values into src values */
int mx_set_row(const double *src, matrix *mx, unsigned short r);

/* function sets column c of matrix mx values into src values */
int mx_set_col(const double *src, matrix *mx, unsigned short c);

/* function sets value val of element(col,row) of matrix *mx*/
int mx_set_cell(double val, matrix *mx, unsigned short row, unsigned short col);

/* function copies row r of matrix mx into dst */
int mx_get_row(double *dst, matrix *mx, unsigned short r);

/* function copies column c of matrix mx into dst */
int mx_get_col(double *dst, matrix *mx, unsigned short c);

/* funcion puts value of element(col, row) from matrix *mx into *dst */
int mx_get_cell(double *dst, const matrix *mx, unsigned short row, unsigned short col);

/* function replaces two rows in a matrix */
int mx_swap_rows(matrix *mx, unsigned short r1, unsigned short r2);

/* function replaces two columns in a matrix */
int mx_swap_cols(matrix *mx, unsigned short c1, unsigned short c2);

/* function changes the size of matrix */
int mx_resize(matrix *mx, unsigned short rows, unsigned short cols);

/* function transponses *mx matrix */
int mx_transpose(matrix *mx);

/* function copies matrix *src into matrix *dst */
int mx_copy(matrix *dst, const matrix *src);

/* function reverses sign of each matrix *mx element*/
int mx_rsign(matrix *mx);

/* function sets 0 into each matrix element if *val is defined */
int mx_clear(matrix *mx);

/* function increases each element of *mx by num */
int mx_add_num(matrix *dst, double num);

/* function multiplies each element of *mx by num */
int mx_mult_num(matrix *dst, double num);

/* function increases each element of r row of *mx by num*/
int mx_row_add_num(matrix *mx, unsigned short r, double num);

/* function multiplies each element of r row of *mx by num */
int mx_row_mult_num(matrix *mx, unsigned short r, double num);

/* function increases each element of r row of *mx by num*/
int mx_col_add_num(matrix *mx, unsigned short c, double num);

/* function multiplies each element of r row of *mx by num */
int mx_col_mult_num(matrix *mx, unsigned short c, double num);

/* function sets *res matrix as result of matrixes *a and *b adding */
int mx_add_matrix(matrix *dst, const matrix *a, const matrix *b);

/* function sets *res matrix as *a subtract *b */
int mx_sub_matrix(matrix *dst, const matrix *a, const matrix *b);

/* function sets *res matrix as result of multiplication of *a by *b */
int mx_mult_matrix(matrix *dst, const matrix *a, const matrix *b);

#endif
